"""
Custom middleware
"""
import time
import logging
import re

from django.conf import settings

from .utils import sql_unmonitoring_this_thread


logger = logging.getLogger(__name__)


class LongRequestMiddleware(object):

    """Long request middleware, remember to put it first
    """

    def __init__(self):
        """Initialize timeout with PATCHY_LONG_REQUEST_TIMEOUT with default to one second
        """
        self.ignore_url_patterns = getattr(settings, 'PATCHY_LONG_REQUEST_IGNORE_URLS', list())
        # skip any sql timeout mornitoring if lr is ignored
        self.stick_to_lr = getattr(settings, 'PATCHY_STICK_TO_LR', True)
        self.timeout = getattr(settings, 'PATCHY_LONG_REQUEST_TIMEOUT', 1)

    def process_request(self, request):
        """record the time in
        """
        self._start = time.time()

        self.url_matched = False
        for url_pattern in self.ignore_url_patterns:
            # if the current path in ignored url list, just ignore it
            if re.match(url_pattern, request.path):
                self.url_matched = True
                if self.stick_to_lr:
                    sql_unmonitoring_this_thread()
                break

    def process_response(self, request, response):
        """record the time out
        """
        self._end = time.time()
        elapsed = self._end - self._start
        if elapsed > self.timeout and not self.url_matched:
            # too long and log to target
            logger.error('[Long Request]Path: %s, Time: %s s' % (request.path, elapsed))

        response['X-ELAPSED'] = elapsed
        return response



import time
import logging
import threading
from functools import wraps

from django.conf import settings
from django.db.backends.utils import CursorWrapper

logger = logging.getLogger(__name__)

_locals = threading.local()


def this_thread_is_sql_monitoring():
    return getattr(_locals, 'sql_monitoring', True)


def sql_monitoring_this_thread():
    _locals.sql_monitoring = True


def sql_unmonitoring_this_thread():
    _locals.sql_monitoring = False


class NoSQLMonitoring(object):

    def __call__(self, func):
        @wraps(func)
        def decorator(*args, **kw):
            with self:
                return func(*args, **kw)
        return decorator

    def __enter__(self):
        _locals.patchy_outer_scope = this_thread_is_sql_monitoring()
        sql_unmonitoring_this_thread()

    def __exit__(self, type, value, tb):
        if _locals.patchy_outer_scope:
            sql_monitoring_this_thread()

no_sql_monitoring = NoSQLMonitoring()


original = CursorWrapper.execute


def long_sql_execute_wrapper(*args, **kwargs):
    TIMEOUT = getattr(settings, 'PATCHY_LONG_SQL_TIMEOUT', 0.05)
    try:
        start = time.time()
        result = original(*args, **kwargs)
        return result
    finally:
        end = time.time()
        duration = end - start
        if duration > TIMEOUT and this_thread_is_sql_monitoring():
            try:
                message = 'SQL: (%s), Args: (%s), Execution time: %.6fs' % (args[1], args[2], duration)
            except IndexError:
                message = 'SQL: (%s), Args: (), Execution time: %.6fs' % (args[1], duration)
            logger.error(message)



import os
import re
from setuptools import setup

try:
    from pypandoc import convert

    def read_md(f):
        return convert(f, 'rst')
except ImportError:
    def read_md(f):
        return open(f, 'r').read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]

version = get_version('patchy')

setup(
    name='djangopatchy',
    version=version,
    license='MIT',
    description='Useful django patch for large project',
    long_description=read_md('README.md'),
    author='Fu Jian',
    author_email='fujian_en@126.com',
    packages=get_packages('patchy'),
    install_requires=[],
    zip_safe=False,
    classifiers=[
        'Framework :: Django',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
